@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2014 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartCV.cxx 2138 2010-10-03 00:23:13Z stephena $
+// $Id: CartCV.cxx 2960 2014-07-24 17:15:28Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -48,9 +48,6 @@ CartridgeCV::CartridgeCV(const uInt8* image, uInt32 size,
     memcpy(myInitialRAM, image, 1024);
   }
   createCodeAccessBase(2048+1024);
-
-  // This cart contains 1024 bytes extended RAM @ 0x1000
-  registerRamArea(0x1000, 1024, 0x00, 0x400);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -90,7 +87,7 @@ void CartridgeCV::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1800 & mask) == 0);
 
-  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+  System::PageAccess access(this, System::PA_READ);
 
   // Map ROM image into the system
   for(uInt32 address = 0x1800; address < 0x2000; address += (1 << shift))
@@ -152,26 +149,6 @@ bool CartridgeCV::poke(uInt16, uInt8)
   return false;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeCV::bank(uInt16 bank)
-{
-  // Doesn't support bankswitching
-  return false;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeCV::bank() const
-{
-  // Doesn't support bankswitching
-  return 0;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt16 CartridgeCV::bankCount() const
-{
-  return 1;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCV::patch(uInt16 address, uInt8 value)
 {
@@ -204,15 +181,11 @@ bool CartridgeCV::save(Serializer& out) const
   try
   {
     out.putString(name());
-
-    // Output RAM
-    out.putInt(1024);
-    for(uInt32 addr = 0; addr < 1024; ++addr)
-      out.putByte((char)myRAM[addr]);
+    out.putByteArray(myRAM, 1024);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeCV::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeCV::save" << endl;
     return false;
   }
 
@@ -227,14 +200,11 @@ bool CartridgeCV::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    // Input RAM
-    uInt32 limit = (uInt32) in.getInt();
-    for(uInt32 addr = 0; addr < limit; ++addr)
-      myRAM[addr] = (uInt8) in.getByte();
+    in.getByteArray(myRAM, 1024);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeCV::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeCV::load" << endl;
     return false;
   }
 
