@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2014 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx 2301 2011-12-28 00:25:19Z stephena $
+// $Id: Settings.cxx 2989 2014-09-03 13:27:33Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -27,45 +27,54 @@
 #include "OSystem.hxx"
 #include "Version.hxx"
 
+#ifdef DEBUGGER_SUPPORT
+  #include "DebuggerDialog.hxx"
+#endif
+
 #include "Settings.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Settings::Settings(OSystem* osystem)
+Settings::Settings(OSystem& osystem)
   : myOSystem(osystem)
 {
-  // Add this settings object to the OSystem
-  myOSystem->attach(this);
-
-  // Add options that are common to all versions of Stella
-  setInternal("video", "soft");
-
-  // OpenGL specific options
-  setInternal("gl_filter", "nearest");
-  setInternal("gl_aspectn", "90");
-  setInternal("gl_aspectp", "100");
-  setInternal("gl_fsmax", "true");
-  setInternal("gl_lib", "libGL.so");
-  setInternal("gl_vsync", "true");
-  setInternal("gl_vbo", "true");
-
-  // Framebuffer-related options
-  setInternal("tia_filter", "zoom2x");
-  setInternal("fullscreen", "0");
-  setInternal("fullres", "auto");
+  // Video-related options
+  setInternal("video", "");
+  setInternal("vsync", "true");
+  setInternal("fullscreen", "false");
   setInternal("center", "false");
-  setInternal("grabmouse", "true");
   setInternal("palette", "standard");
   setInternal("colorloss", "true");
   setInternal("timing", "sleep");
   setInternal("uimessages", "true");
 
+  // TIA specific options
+  setInternal("tia.zoom", "2");
+  setInternal("tia.inter", "false");
+  setInternal("tia.aspectn", "90");
+  setInternal("tia.aspectp", "100");
+  setInternal("tia.fsfill", "false");
+
+  // TV filtering options
+  setInternal("tv.filter", "0");
+  setInternal("tv.scanlines", "25");
+  setInternal("tv.scaninter", "true");
+  // TV options when using 'custom' mode
+  setInternal("tv.contrast", "0.0");
+  setInternal("tv.brightness", "0.0");
+  setInternal("tv.hue", "0.0");
+  setInternal("tv.saturation", "0.0");
+  setInternal("tv.gamma", "0.0");
+  setInternal("tv.sharpness", "0.0");
+  setInternal("tv.resolution", "0.0");
+  setInternal("tv.artifacts", "0.0");
+  setInternal("tv.fringing", "0.0");
+  setInternal("tv.bleed", "0.0");
+
   // Sound options
   setInternal("sound", "true");
   setInternal("fragsize", "512");
   setInternal("freq", "31400");
-  setInternal("tiafreq", "31400");
   setInternal("volume", "100");
-  setInternal("clipvol", "true");
 
   // Input event options
   setInternal("keymap", "");
@@ -73,15 +82,18 @@ Settings::Settings(OSystem* osystem)
   setInternal("combomap", "");
   setInternal("joydeadzone", "13");
   setInternal("joyallow4", "false");
-  setInternal("mcontrol", "auto");
+  setInternal("usemouse", "analog");
+  setInternal("grabmouse", "true");
+  setInternal("hidecursor", "false");
   setInternal("dsense", "5");
   setInternal("msense", "7");
-  setInternal("sa1", "left");
-  setInternal("sa2", "right");
+  setInternal("saport", "lr");
   setInternal("ctrlcombo", "true");
 
   // Snapshot options
-  setInternal("snapdir", "");
+  setInternal("snapsavedir", "");
+  setInternal("snaploaddir", "");
+  setInternal("snapname", "int");
   setInternal("sssingle", "false");
   setInternal("ss1x", "false");
   setInternal("ssinterval", "2");
@@ -92,20 +104,24 @@ Settings::Settings(OSystem* osystem)
   setInternal("cheatfile", "");
   setInternal("palettefile", "");
   setInternal("propsfile", "");
-  setInternal("eepromdir", "");
+  setInternal("nvramdir", "");
   setInternal("cfgdir", "");
 
   // ROM browser options
-  setInternal("uselauncher", "true");
-  setInternal("launcherres", "640x480");
+  setInternal("exitlauncher", "false");
+  setInternal("launcherres", GUI::Size(640, 480));
   setInternal("launcherfont", "medium");
   setInternal("launcherexts", "allroms");
   setInternal("romviewer", "0");
   setInternal("lastrom", "");
 
   // UI-related options
-  setInternal("debuggerres", "1030x690");
-  setInternal("uipalette", "0");
+#ifdef DEBUGGER_SUPPORT
+  setInternal("dbg.res",
+    GUI::Size(DebuggerDialog::kMediumFontMinW,
+              DebuggerDialog::kMediumFontMinH));
+#endif
+  setInternal("uipalette", "standard");
   setInternal("listdelay", "300");
   setInternal("mwheel", "4");
 
@@ -114,6 +130,7 @@ Settings::Settings(OSystem* osystem)
   setInternal("loglevel", "1");
   setInternal("logtoconsole", "0");
   setInternal("tiadriven", "false");
+  setInternal("cpurandom", "true");
   setInternal("ramrandom", "true");
   setInternal("avoxport", "");
   setInternal("stats", "false");
@@ -121,13 +138,20 @@ Settings::Settings(OSystem* osystem)
   setExternal("romloadcount", "0");
   setExternal("maxres", "");
 
-  // Debugger options
-  setInternal("resolvedata", "auto");
-  setInternal("gfxformat", "2");
-  setInternal("showaddr", "true");
+#ifdef DEBUGGER_SUPPORT
+  // Debugger/disassembly options
+  setInternal("dbg.fontstyle", "0");
+  setInternal("dbg.uhex", "true");
+  setInternal("dis.resolve", "true");
+  setInternal("dis.gfxformat", "2");
+  setInternal("dis.showaddr", "true");
+  setInternal("dis.relocate", "false");
+#endif
 
+#ifdef DTHUMB_SUPPORT
   // Thumb ARM emulation options
   setInternal("thumb.trapfatal", "true");
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -143,10 +167,10 @@ void Settings::loadConfig()
   string line, key, value;
   string::size_type equalPos, garbage;
 
-  ifstream in(myOSystem->configFile().c_str());
+  ifstream in(myOSystem.configFile().c_str());
   if(!in || !in.is_open())
   {
-    myOSystem->logMessage("ERROR: Couldn't load settings file\n", 0);
+    myOSystem.logMessage("ERROR: Couldn't load settings file", 0);
     return;
   }
 
@@ -201,9 +225,10 @@ string Settings::loadCommandLine(int argc, char** argv)
         return "";
       }
 
-      // Take care of arguments without an option
+      // Take care of arguments without an option or ones that shouldn't
+      // be saved to the config file
       if(key == "rominfo" || key == "debug" || key == "holdreset" ||
-         key == "holdselect" || key == "holdbutton0" || key == "takesnapshot")
+         key == "holdselect" || key == "takesnapshot")
       {
         setExternal(key, "true");
         continue;
@@ -213,7 +238,7 @@ string Settings::loadCommandLine(int argc, char** argv)
       if(++i >= argc)
       {
         buf << "Missing argument for '" << key << "'" << endl;
-        myOSystem->logMessage(buf.str(), 0);
+        myOSystem.logMessage(buf.str(), 0);
         return "";
       }
       string value = argv[i];
@@ -234,7 +259,7 @@ string Settings::loadCommandLine(int argc, char** argv)
         buf << "(E)\n";
       }
 
-      myOSystem->logMessage(buf.str(), 2);
+      myOSystem.logMessage(buf.str(), 2);
     }
     else
       return key;
@@ -249,43 +274,29 @@ void Settings::validate()
   string s;
   int i;
 
-  s = getString("video");
-  if(s != "soft" && s != "gl")  setInternal("video", "soft");
-
   s = getString("timing");
   if(s != "sleep" && s != "busy")  setInternal("timing", "sleep");
 
-#ifdef DISPLAY_OPENGL
-  s = getString("gl_filter");
-  if(s != "linear" && s != "nearest")  setInternal("gl_filter", "nearest");
+  i = getInt("tia.aspectn");
+  if(i < 80 || i > 120)  setInternal("tia.aspectn", "90");
+  i = getInt("tia.aspectp");
+  if(i < 80 || i > 120)  setInternal("tia.aspectp", "100");
 
-  i = getInt("gl_aspectn");
-  if(i < 80 || i > 120)  setInternal("gl_aspectn", "100");
-  i = getInt("gl_aspectp");
-  if(i < 80 || i > 120)  setInternal("gl_aspectp", "100");
-#endif
+  i = getInt("tv.filter");
+  if(i < 0 || i > 5)  setInternal("tv.filter", "0");
 
 #ifdef SOUND_SUPPORT
   i = getInt("volume");
   if(i < 0 || i > 100)    setInternal("volume", "100");
   i = getInt("freq");
-  if(i < 0 || i > 48000)  setInternal("freq", "31400");
-  i = getInt("tiafreq");
-  if(i < 0 || i > 48000)  setInternal("tiafreq", "31400");
+  if(!(i == 11025 || i == 22050 || i == 31400 || i == 44100 || i == 48000))
+    setInternal("freq", "31400");
 #endif
 
   i = getInt("joydeadzone");
   if(i < 0)        setInternal("joydeadzone", "0");
   else if(i > 29)  setInternal("joydeadzone", "29");
 
-  s = getString("mcontrol");
-  if(s != "auto")
-  {
-    // Note: these constants are from Controller::MouseAxisType enum
-    if(s.length() != 2 || s[0] < '0' || s[0] > '5' || s[1] < '0' || s[1] > '5')
-      setInternal("mcontrol", "auto");
-  }
-
   if(i < 1)        setInternal("dsense", "1");
   else if(i > 10)  setInternal("dsense", "10");
 
@@ -331,26 +342,17 @@ void Settings::usage()
     << "Valid options are:" << endl
     << endl
     << "  -video        <type>         Type is one of the following:\n"
-    << "                 soft            SDL software mode\n"
-  #ifdef DISPLAY_OPENGL
-    << "                 gl              SDL OpenGL mode\n"
-    << endl
-    << "  -gl_lib       <name>         Specify the OpenGL library\n"
-    << "  -gl_filter    <type>         Type is one of the following:\n"
-    << "                 nearest         Normal scaling (GL_NEAREST)\n"
-    << "                 linear          Blurred scaling (GL_LINEAR)\n"
-    << "  -gl_aspectn   <number>       Scale the TIA width by the given percentage in NTSC mode\n"
-    << "  -gl_aspectp   <number>       Scale the TIA width by the given percentage in PAL mode\n"
-    << "  -gl_fsmax     <1|0>          Stretch GL image in fullscreen emulation mode\n"
-    << "  -gl_vsync     <1|0>          Enable 'synchronize to vertical blank interrupt'\n"
-    << "  -gl_vbo       <1|0>          Enable 'vertex buffer objects'\n"
-    << endl
+  #ifdef BSPF_WINDOWS
+    << "                 direct3d        Direct3D acceleration\n"
   #endif
-    << "  -tia_filter   <filter>       Use the specified filter in emulation mode\n"
-    << "  -fullscreen   <1|0|-1>       Use fullscreen mode (1 or 0), or disable switching to fullscreen entirely\n"
-    << "  -fullres      <auto|WxH>     The resolution to use in fullscreen mode\n"
+    << "                 opengl          OpenGL acceleration\n"
+    << "                 opengles2       OpenGLES 2 acceleration\n"
+    << "                 opengles        OpenGLES 1 acceleration\n"
+    << "                 software        Software mode (no acceleration)\n"
+    << endl
+    << "  -vsync        <1|0>          Enable 'synchronize to vertical blank interrupt'\n"
+    << "  -fullscreen   <1|0>          Enable fullscreen mode\n"
     << "  -center       <1|0>          Centers game window (if possible)\n"
-    << "  -grabmouse    <1|0>          Keeps the mouse in the game window\n"
     << "  -palette      <standard|     Use the specified color palette\n"
     << "                 z26|\n"
     << "                 user>\n"
@@ -362,34 +364,57 @@ void Settings::usage()
   #ifdef SOUND_SUPPORT
     << "  -sound        <1|0>          Enable sound generation\n"
     << "  -fragsize     <number>       The size of sound fragments (must be a power of two)\n"
-    << "  -freq         <number>       Set sound sample output frequency (0 - 48000)\n"
-    << "  -tiafreq      <number>       Set sound sample generation frequency (0 - 48000)\n"
+    << "  -freq         <number>       Set sound sample output frequency (11025|22050|31400|44100|48000)\n"
     << "  -volume       <number>       Set the volume (0 - 100)\n"
-    << "  -clipvol      <1|0>          Enable volume clipping (eliminates popping)\n"
     << endl
   #endif
+    << "  -tia.zoom     <zoom>         Use the specified zoom level (windowed mode) for TIA image\n"
+    << "  -tia.inter    <1|0>          Enable interpolated (smooth) scaling for TIA image\n"
+    << "  -tia.aspectn  <number>       Scale TIA width by the given percentage in NTSC mode\n"
+    << "  -tia.aspectp  <number>       Scale TIA width by the given percentage in PAL mode\n"
+    << "  -tia.fsfill   <1|0>          Stretch TIA image to fill fullscreen mode\n"
+    << endl
+    << "  -tv.filter    <0-5>          Set TV effects off (0) or to specified mode (1-5)\n"
+    << "  -tv.scanlines <0-100>        Set scanline intensity to percentage (0 disables completely)\n"
+    << "  -tv.scaninter <1|0>          Enable interpolated (smooth) scanlines\n"
+    << "  -tv.contrast    <value>      Set TV effects custom contrast to value 1.0 - 1.0\n"
+    << "  -tv.brightness  <value>      Set TV effects custom brightness to value 1.0 - 1.0\n"
+    << "  -tv.hue         <value>      Set TV effects custom hue to value 1.0 - 1.0\n"
+    << "  -tv.saturation  <value>      Set TV effects custom saturation to value 1.0 - 1.0\n"
+    << "  -tv.gamma       <value>      Set TV effects custom gamma to value 1.0 - 1.0\n"
+    << "  -tv.sharpness   <value>      Set TV effects custom sharpness to value 1.0 - 1.0\n"
+    << "  -tv.resolution  <value>      Set TV effects custom resolution to value 1.0 - 1.0\n"
+    << "  -tv.artifacts   <value>      Set TV effects custom artifacts to value 1.0 - 1.0\n"
+    << "  -tv.fringing    <value>      Set TV effects custom fringing to value 1.0 - 1.0\n"
+    << "  -tv.bleed       <value>      Set TV effects custom bleed to value 1.0 - 1.0\n"
+    << endl
     << "  -cheat        <code>         Use the specified cheatcode (see manual for description)\n"
     << "  -loglevel     <0|1|2>        Set level of logging during application run\n"
     << "  -logtoconsole <1|0>          Log output to console/commandline\n"
     << "  -joydeadzone  <number>       Sets 'deadzone' area for analog joysticks (0-29)\n"
     << "  -joyallow4    <1|0>          Allow all 4 directions on a joystick to be pressed simultaneously\n"
-    << "  -mcontrol     <auto|XY>      Use mouse axes as specified paddle controller type (see manual)\n"
+    << "  -usemouse     <always|\n"
+    << "                 analog|\n"
+    << "                 never>        Use mouse as a controller as specified by ROM properties in given mode(see manual)\n"
+    << "  -grabmouse    <1|0>          Locks the mouse cursor in the TIA window\n"
+    << "  -hidecursor   <1|0>          Always hide the cursor, or show it when appropriate\n"
     << "  -dsense       <number>       Sensitivity of digital emulated paddle movement (1-10)\n"
     << "  -msense       <number>       Sensitivity of mouse emulated paddle movement (1-15)\n"
-    << "  -sa1          <left|right>   Stelladaptor 1 emulates specified joystick port\n"
-    << "  -sa2          <left|right>   Stelladaptor 2 emulates specified joystick port\n"
+    << "  -saport       <lr|rl>        How to assign virtual ports to multiple Stelladaptor/2600-daptors\n"
     << "  -ctrlcombo    <1|0>          Use key combos involving the Control key (Control-Q for quit may be disabled!)\n"
     << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
     << "  -stats        <1|0>          Overlay console info during emulation\n"
     << "  -fastscbios   <1|0>          Disable Supercharger BIOS progress loading bars\n"
-    << "  -snapdir      <path>         The directory to save snapshot files to\n"
+    << "  -snapsavedir  <path>         The directory to save snapshot files to\n"
+    << "  -snaploaddir  <path>         The directory to load snapshot files from\n"
+    << "  -snapname     <int|rom>      Name snapshots according to internal database or ROM\n"
     << "  -sssingle     <1|0>          Generate single snapshot instead of many\n"
     << "  -ss1x         <1|0>          Generate TIA snapshot in 1x mode (ignore scaling/effects)\n"
     << "  -ssinterval   <number        Number of seconds between snapshots in continuous snapshot mode\n"
     << endl
     << "  -rominfo      <rom>          Display detailed information for the given ROM\n"
     << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
-    << "  -uselauncher  <1|0>          Use the built-in ROM launcher\n"
+    << "  -exitlauncher <1|0>          On exiting a ROM, go back to the ROM launcher\n"
     << "  -launcherres  <WxH>          The resolution to use in ROM launcher mode\n"
     << "  -launcherfont <small|medium| Use the specified font in the ROM launcher\n"
     << "                 large>\n"
@@ -397,34 +422,38 @@ void Settings::usage()
     << "                 allroms|        (exts is a ':' separated list of extensions)\n"
     << "                 exts\n"
     << "  -romviewer    <0|1|2>        Show ROM info viewer at given zoom level in ROM launcher (0 for off)\n"
-    << "  -uipalette    <1|2>          Used the specified palette for UI elements\n"
     << "  -listdelay    <delay>        Time to wait between keypresses in list widgets (300-1000)\n"
     << "  -mwheel       <lines>        Number of lines the mouse wheel will scroll in UI\n"
-    << "  -statedir     <dir>          Directory in which to save state files\n"
+    << "  -romdir       <dir>          Directory in which to load ROM files\n"
+    << "  -statedir     <dir>          Directory in which to save/load state files\n"
     << "  -cheatfile    <file>         Full pathname of cheatfile database\n"
     << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
     << "  -propsfile    <file>         Full pathname of ROM properties file\n"
-    << "  -eepromdir    <dir>          Directory in which to save EEPROM files\n"
+    << "  -nvramdir     <dir>          Directory in which to save/load flash/EEPROM files\n"
     << "  -cfgdir       <dir>          Directory in which to save Distella config files\n"
     << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
-    << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
     << "  -holdreset                   Start the emulator with the Game Reset switch held down\n"
     << "  -holdselect                  Start the emulator with the Game Select switch held down\n"
-    << "  -holdbutton0                 Start the emulator with the left joystick button held down\n"
+    << "  -holdjoy0     <U,D,L,R,F>    Start the emulator with the left joystick direction/fire button held down\n"
+    << "  -holdjoy1     <U,D,L,R,F>    Start the emulator with the right joystick direction/fire button held down\n"
     << "  -tiadriven    <1|0>          Drive unused TIA pins randomly on a read/peek\n"
+    << "  -cpurandom    <1|0>          Randomize the contents of CPU registers on reset\n"
     << "  -ramrandom    <1|0>          Randomize the contents of RAM on reset\n"
+    << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
     << "  -help                        Show the text you're now reading\n"
   #ifdef DEBUGGER_SUPPORT
     << endl
     << " The following options are meant for developers\n"
     << " Arguments are more fully explained in the manual\n"
     << endl
-    << "   -resolvedata  <never|       Set automatic code vs. data determination in the disassembler\n"
-    << "                  always|auto>\n"
-    << "   -gfxformat    <2|16>        Set base to use for displaying GFX sections in the disassembler\n"
-    << "   -showaddr     <1|0>         Show opcode addresses in the disassembler\n"
-    << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
-    << "   -break        <address>     Set a breakpoint at 'address'\n"
+    << "   -dis.resolve   <1|0>        Attempt to resolve code sections in disassembler\n"
+    << "   -dis.gfxformat <2|16>       Set base to use for displaying GFX sections in disassembler\n"
+    << "   -dis.showaddr  <1|0>        Show opcode addresses in disassembler\n"
+    << "   -dis.relocate  <1|0>        Relocate calls out of address range in disassembler\n"
+    << endl
+    << "   -dbg.res       <WxH>        The resolution to use in debugger mode\n"
+    << "   -dbg.fontstyle <0-3>        Font style to use in debugger window (bold vs. normal)\n"
+    << "   -break         <address>    Set a breakpoint at 'address'\n"
     << "   -debug                      Start in debugger mode\n"
     << endl
     << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
@@ -447,6 +476,28 @@ void Settings::usage()
     << endl << flush;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const Variant& Settings::value(const string& key) const
+{
+  // Try to find the named setting and answer its value
+  int idx = -1;
+  if((idx = getInternalPos(key)) != -1)
+    return myInternalSettings[idx].value;
+  else if((idx = getExternalPos(key)) != -1)
+    return myExternalSettings[idx].value;
+  else
+    return EmptyVariant;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::setValue(const string& key, const Variant& value)
+{
+  if(int idx = getInternalPos(key) != -1)
+    setInternal(key, value, idx);
+  else
+    setExternal(key, value);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Settings::saveConfig()
 {
@@ -465,10 +516,10 @@ void Settings::saveConfig()
   if(!settingsChanged)
     return;
 
-  ofstream out(myOSystem->configFile().c_str());
+  ofstream out(myOSystem.configFile().c_str());
   if(!out || !out.is_open())
   {
-    myOSystem->logMessage("ERROR: Couldn't save settings file\n", 0);
+    myOSystem.logMessage("ERROR: Couldn't save settings file", 0);
     return;
   }
 
@@ -497,139 +548,6 @@ void Settings::saveConfig()
   out.close();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setInt(const string& key, const int value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setFloat(const string& key, const float value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setBool(const string& key, const bool value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setString(const string& key, const string& value)
-{
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, value, idx);
-  else
-    setExternal(key, value);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::getSize(const string& key, int& x, int& y) const
-{
-  char c = '\0';
-  x = y = -1;
-  string size = getString(key);
-  istringstream buf(size);
-  buf >> x >> c >> y;
-  if(c != 'x')
-    x = y = -1;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::getInt(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return (int) atoi(myInternalSettings[idx].value.c_str());
-  else if((idx = getExternalPos(key)) != -1)
-    return (int) atoi(myExternalSettings[idx].value.c_str());
-  else
-    return -1;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-float Settings::getFloat(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return (float) atof(myInternalSettings[idx].value.c_str());
-  else if((idx = getExternalPos(key)) != -1)
-    return (float) atof(myExternalSettings[idx].value.c_str());
-  else
-    return -1.0;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Settings::getBool(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-  {
-    const string& value = myInternalSettings[idx].value;
-    if(value == "1" || value == "true")
-      return true;
-    else if(value == "0" || value == "false")
-      return false;
-    else
-      return false;
-  }
-  else if((idx = getExternalPos(key)) != -1)
-  {
-    const string& value = myExternalSettings[idx].value;
-    if(value == "1" || value == "true")
-      return true;
-    else if(value == "0" || value == "false")
-      return false;
-    else
-      return false;
-  }
-  else
-    return false;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& Settings::getString(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return myInternalSettings[idx].value;
-  else if((idx = getExternalPos(key)) != -1)
-    return myExternalSettings[idx].value;
-  else
-    return EmptyString;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setSize(const string& key, const int value1, const int value2)
-{
-  ostringstream buf;
-  buf << value1 << "x" << value2;
-  setString(key, buf.str());
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInternalPos(const string& key) const
 {
@@ -651,7 +569,7 @@ int Settings::getExternalPos(const string& key) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::setInternal(const string& key, const string& value,
+int Settings::setInternal(const string& key, const Variant& value,
                           int pos, bool useAsInitial)
 {
   int idx = -1;
@@ -706,7 +624,7 @@ int Settings::setInternal(const string& key, const string& value,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::setExternal(const string& key, const string& value,
+int Settings::setExternal(const string& key, const Variant& value,
                           int pos, bool useAsInitial)
 {
   int idx = -1;
@@ -759,7 +677,8 @@ int Settings::setExternal(const string& key, const string& value,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Settings::Settings(const Settings&)
+Settings::Settings(const Settings& s)
+  : myOSystem(s.myOSystem)
 {
 }
 
